   NB. The Role of Composition in Computer Programming
   NB. APL95 Conference Proceedings, San Antonio, Texas.  June 1995
   NB. Copyright ACM                Donald B. McIntyre

   (-:@-)  0 1 2 3             NB. Compose a new function
   -:@-  0 1 2 3               NB. Parentheses not necessary
   h=.+:@+                     NB. Name a composite function
   g=. >:                      NB. Increment
   f=. *:                      NB. Square
   h=. g@f                     NB. Mathematics Dictionary (1992)
   e=. f@g                     NB. h and e are not the same
   h 2
   e 2
   v=. h"0                     NB. (verb - conjunction - noun) is verb
   y=. h x=. i.5
   v D.1 x                     NB. First derivative of function v
   h"0                         NB. Verb
   D.                          NB. Conjunction
   d=. h"0 D.                  NB. Adverb
   (1 d)                       NB. Verb: First derivative
   (2 d)                       NB. Verb: Second derivative
   x,y,(1 d y),: 2 d y         NB. Table of 1st and 2nd derivatives
   y=. i.3 4  [  n=. 1 1 ; 2 2; 0 3
   amend=. n}                  NB. (noun - adverb) is verb
   97 98 99 amend y
   %&180 (30 60 90)            NB. Divide with (by) 180

   NB. COMPOSITIONS OF 2 OR 3 VERBS
   f=. %  [. g=. +  [. h=. *
   x=. 2 [ y=. 5

   NB. COMPOSITION OF 2 VERBS USED AS DYAD
   x g@h y                     NB. Conjunction:  Atop
   g (x h y)
   x g&h y                     NB. Conjunction:  With
   (h x) g (h y)
   x (g h) y                   NB. Hook
   x g (h y)

   NB. COMPOSITION OF 2 VERBS USED AS MONAD
   g@h y
   g (h y)
   g&h y
   g (h y)
   (g h) y
   y g (h y)

   NB. COMPOSITION OF 3 VERBS:  FORKS
   x (f g h) y               NB. dyad
   (x f y) g (x h y)
   (f g h) y                 NB. Monad
   (f y) g (h y)

   NB. Some variants
   (] g h) y
   (g h) y
   x ([ g h@]) y
   x (g h) y
   x (] g h) y
   y g (x h y)
   x ([ g h) y
   x g ( x h y)
   x (f@[ g h@]) y
   (f x) g (h y)

   NB. Conjunctions have long left-scope and short right-scope
   f@g@h
   f@(g@h)             NB. Parentheses control order of execution

   tree=. 5!:4@<        NB. Tree display
   tree 'tree'
   e=. f@g@h
   tree 'e'

   linear=. 5!:5@<               NB. Linear display
   erase=. 4!:55@;:              NB. Erase objects
   setrl=. 9!:1                  NB. Set random link
   nc=. 4!:0@<                   NB. Name class

   NB. TRAINS
   e=. [: g h  [.  f=. g@h       NB. Similarities and differences
   g=. +/      [.  h=. +/"1
   y=. i.3 4
   g h y
   h g y
   e y
   f y

   g@h b. 0            NB. Adverb b. can be used to show ranks
   p=. g@:h            NB. Ranks of conjunction @: (AT) are unbounded
   p y

   abs=. | : [:        NB. Absolute value. Monad only
   res=. [: : |        NB. Residue. Dyad only

   NB. COMPOSITION BY FORKS AND CONJUNCTIONS
   mean=. +/ % #
   tree 'mean'

   g h i j k           NB. Train with an odd number of verbs
   f g h i j k         NB. Train with an even number of verbs

   rss=. %:@(+/@*:)          NB. Root sum of squares
   NB. [: caps a fork, making the verb to its right a monad
   rss=. [: %: +/@*:         NB. A fork
   rss
   rss 3 4
   rss=. [: %: [: +/ *:      NB. A train of 5 verbs
   rss=. +/&.*:              NB. Conjunction &. Under
   rss 3 4

   NB. A TRUTH TABLE AS A FORK
   2 2 2 #: i. 2^3
   tt=. #&2 #: i.@(2&^)
   tt=. #&2 #: [: i. 2&^                NB. train of 5 verbs
   tt=. (] # 2:) #: [: i. 2: ^ ]        NB. train of 7 verbs
   tt

   s=. '(y. # 2) #: i. 2^y.'            NB. Character string
   e=. 3 : s                            NB. Explicit definition
   t=. 13 : s                           NB. Tacit definition

   linear 't'
   tt=. #:@i.@(2&^)               NB. Conjunctions
   tt=. [: #: [: i. 2: ^ ]        NB. Train of 7 verbs
   tt 3

   NB. HOOKS
   NB. BORDERING A TABLE
   h=. , mean
   h y=. i.3 4      NB. Append column means
   h"1 y            NB. Append mean to the rank-1 cells of y

   cols=. ,: +/&.>              NB. A hook with Under  &.
   rows=. ; ,.@:(+/"1)          NB. A hook with At     @:
   rows
   $ ,.@(+/"1)  y               NB. Using @
   $ ,.@:(+/"1) y               NB. Using @:

   rows=. ; [: ,. +/"1     NB. Train of 4 verbs (Hook with a fork tine
   rows
   totals=. cols@rows
   totals y

   NB. Select non-negative items
   setrl 7^5
   ]y=. 10-~?10#20
   h=. >:&0
   h=. >: 0:     NB. Alternative form
   (h y) # y
   y #~ h y                NB. Commute adverb
   select=. #~ h           NB. Hook
   select  y
   select=. (h # ])        NB. Fork
   select y

   NB. A MARKOV CHAIN EXAMPLE
y=.0 11 36 21 52;28  0  4  4  0;34  2  0 45 13;29  1 45  0  3;28 23  9 8  0
   ]m =. >y
   diag=. (<0 1)&|:               NB. Diagonal
   ixd=. [: <"1 $ #: diag@i.@$    NB. Indices of diagonal elements
   ix=. ixd@]
   ix m

   amend=. [`ix`]}                NB. (Gerund - Adverb) is Noun
   n=. 1000 amend m
   g=. *:@(+/"1) % +/@,           NB. Fork
   5 ": z=. (amend~ g) n          NB. Hook.  Format a table

   NB. SURFACE AREAS AND VOLUMES OF SPHERES
   surface=. 4&*@o.@*:            NB. Conjunctions
   surface=. 4: * [: o. *:        NB. Train of 5 verbs
   surface 2 3 4 5
   vol=. 4r3&*@o.@(^&3)           NB. 4r3 is a rational number
   vol=. 4r3"_ * [: o. ^&3        NB. Train of 5 verbs
   4r3"_  9                       NB. 4r3"_ is a verb returning 4 over 3
   vol 2 3 4 5

   NB. HERON'S FORMULA FOR THE AREA OF AN OBLIQUE TRIANGLE
   s=. -:@(+/)                    NB. s is half the sum
   s=. [: -: +/                   NB. Avoids parentheses
   h=. -~ s                       NB. Hook, commuting the arguments
   h=. s - ]                      NB. Fork
   g=. s , s - ]                  NB. Train of 5 verbs
   area=. %:@(*/@(s , s - ]))
   area=. [: %: [: */ s , s - ]   NB. Train of 9 verbs
   area 15 17 20

   NB. ANGLES OF A TRIANGLE WHOSE SIDES ARE GIVEN
   rfd=. %&180@o.                 NB. Radians from degrees
   rfd=. o. % 180"_               NB. 180"_ is a verb
   dfr=. rfd^:_1                  NB. Degrees from radians
   arctan=. dfr@(_3&o.)           NB. Arctangent
   arctan=. [: dfr _3"_ o. ]      NB. Train of 5 verbs

   erase 'n'    
   r=. %:@(numerator % denominator)
   r=. [: %: n % s                NB. Train of 5 verbs
   n=. */@(s - ])
   n=. [: */ s - ]                NB. Train of 5 verbs
   tanA2=. r % s - {.             NB. Train of 5 verbs
   A=. +:@arctan@tanA2            NB. Conjunctions
   A=. [: +: [: arctan tanA2      NB. Train of 5 verbs
   A 0 1 2 |."0 1 y=. 15 17 20    NB. Apply A to a cyclic rotation of y
   f=. [: A 0 1 2&|."0 1
   NB. f y      gives a length error

   e=. 3 : 'A 0 1 2 |."0 1 y.'        NB. Explicit definition
   e y
   g=. 13 : 'A 0 1 2 |."0 1 y.'       NB. Tacit definition
   linear 'g'
   angles=. [: A 0 1 2"_ |."0 1 ]
   angles                             NB. Display show train of 5 verbs
   angles y

   NB. FIND SUBSTRINGS COMMON TO TWO STRINGS
   x=. 0 1 2  [ y=. 4 5 1 2
   equ=. =/ |.
   x equ y
   ]b=. x [/.@equ y
   b #&.>"1 x
   ]m=. >:i.3 4
   ]/. m

   x=. 1 2 3 [ y=. 2 3 5 4
   x equ y
   b=. x ]/.@equ y
   b #&.>"1 x     NB. Incorrectly reports finding the substring 1 2

   equ=. (=/ |.) ,"1 #@[ # 0:     NB. Pad with zeros
   x equ y
   x=. 1 2 3 0 5 6
   y=. 1 2 8 0 5 6 3 0
   x (=/ |.) y                    NB. Two substrings on one diagonal
   h=. [/.@equ #&.>"1 [           NB. Fork
   ]&.> i.2 3
   g=. ,@((<$0)&,.)               NB. Append a column of empty cells
   f=. <;._1                      NB. Cut using empty cells as frets
   e=. -.&(<$0)
   x e@f@g@h y
   ss=. ;&.>@e@f@g@h              NB. Raze
   x ss y                         NB. Substrings in common
   y ss x

   NB. FIND THE PIVOT OF A MATRIX
   utm=. [: , </~@i.@#            NB. Upper triangle mask
   ut=. utm # ,                   NB. Upper triangle
   ixp=. $ #: (i. >./)@|@ut { utm # ,@i.@$    NB. Indices of pivot
   pivot=. <@ixp { ]              NB. Signed value of the pivot

   perm=. 0 0&{ ; ] ; |. ; 1 1&{     NB. Permutations for amend
   ixa=. perm@ixp@{.@]               NB. Indices for amend
   imat=. {:@]                       NB. Amend the last item
   amend=. [`ixa`imat }              NB. (Gerund - Adverb) is verb

   setrl 7^5                         NB. Creat a test matrix
   smatrix=. |: +/ .* ]              NB. See Section 7.6                        
   t=. smatrix 100%~ 50-~ ?6 6$100
   y=. _0.5 (0 0; 2 4; 4 2)} t
   6.2 ": y
   pivot y
   ixp y
   96 97 98 99 amend y ,: =i.#y       NB. Amend identity matrix

   utm y
   ut y
   ixa y,:=i.#y
   imat y,:=i.#y

   NB. DETERMINANTS AND THE AREA OF A TRIANGLE

   ip=. +/ . *               NB. Inner product (with conjunction)
   det=. -/ . *              NB. Determinant
   smatrix=. |: ip ]         NB. Symmetric matrix
   smatrix i.3 4

   a=.0.5 1.25 [ b=.4.4 2.45 [ c=.2.6 0.2   NB. 3 points
   (,"1 1:) t=. > a;b;c         NB. Hook
   area=. -:@det@(,"1 1:)

   area > a;b;c
   area > a;c;b

   NB. COMPOSITION OF ADVERBS
   a=. "_                     NB. Adverb
   g=. [: i. 6:               NB. Constant function. Monad
   h=. (i.2 3) a              NB. Constant function. Dyad
   f=. g : h                  NB. Constant function. Ambi-valent
   f 7
   8 f 10
   x=. 0 1 2 3
   (1*x^2) + (_1*x^1) + (_2*x^0)     NB. Polynomial
   ff=. #.&1 _1 _2 "0         NB. Tacit definition
   ff 2                       NB. 2 is a root
   D=. D.1                    NB. (Conjunction - Noun) is Adverb
   x - (ff x) % ff D x                   NB. Newton-Raphson step
   e=. 3 : 'y. - (ff y.) % (ff D y.)'    NB. Explicit definition
   t=. ] - ff % ff D                     NB. Tacit definition
   t t t x                        NB. 3 applications of t
   t^:3 x                         NB. t "to the power" 3
   t ^:_ x                        NB. Limit
   Newton=. 1 : '] - x. % x. D'   NB. Adverb.  x. stands for verb
   t=. 11 : '] - x. % x. D'       NB. Tacit definition of adverb
   linear 't'
   limit=. ^:_                NB. (Conjunction - Noun) is Adverb
   N=. Newton limit           NB. (Adverb - Adverb) is Adverb
   ff N _2 + i.6              NB. Try 6 different starting points
   ff _1 2                    NB. _1 and 2 are roots

   NB. Norman Thomson's functions f and g
   f=. [: <: *:@{. * {:
   g=. 2&o.@{. +  1&o.@{:
   h=. f , g              NB. A  fork

   y=. 5 0                NB. A starting point on the x-y plane
   y - (h y) (%.|:) h  D y            NB. (a) Immediate execution
   e=. 3 : 'y. - (h y.)(%.|:)h D y.'  NB. (b) Explicit definition
   t=. ] - h (%.|:) h D               NB. (c) Tacit definition
   a=. 1 : '] - x. (%.|:) x. D'       NB. (d) Adverb (x. for the verb)

   y=.>5 0;1 1;1 4;0.5 4.5;0.5 4.8;1 5    NB. 6 starting points
   NB. Find the 4 points of intersection on Thomason's graph
   7.2": y,"1 h a limit ("1) y

   NB. COMPOSITION OF CONJUNCTIONS
   f=. 1&o.                         NB. Sine
   g=. ]                            NB. Returns its argument
   s=. 10 20                        NB. Dimension of the plot
   x=. -:i.8                        NB. Argument
   u0=. >./ - ]                     NB. Verb
   u=. u0@                          NB. Adverb
   (u0@f x) -: f u x                NB. These match
   v0=. ] - <./                     NB. Verb
   v=. v0@                          NB. Adverb
   z0=. (f u x) ,: (g v x)          NB. Scale smallest value to 0
   a=. z0 % >./"1 z0                NB. Immediate execution
   scale=. ] % >./"1                NB. Fork
   a -: scale (f u x) ,: (g v x)    NB. These match

   NB. Conjunction. Explicit definition
   N2=. 2 : 'scale@(x. u ,: y. v)@]'
   a -: f N2 g x                          NB. These match
   N=. scale@(([. u) ,: (]. v))@]         NB. Tacit definition
   N
   NB. Automatic translation from explicit to tacit definition
   N12=. 12 : 'scale@(x. u ,: y. v)@]'

   NB. Produce a graph
   round=. <.@+&0.5
   r=. [ #. [: |:@round <:@[ * ]
   ]b=. s r a
   h=. {&'.*'
   ]z=. h (i. s) e. b

   VS=. 12 : '[: h i.@[ e. [ r x. N y.'    NB. Conjunction
   z-: s f VS g x       NB. These match.  f versus g

   NB. A fuller plot is given by
   NB. 25 90 f VS g 4%~ i.60

   NB. HIDDEN LINE PROBLEM
   a=.0.5 1.25 [ b=.4.4 2.45 [ c=.2.6 0.2
 p=. > 3.5 5.1; 7 2.2; 1.3 3.6;3.3 2.9; 0.2 _0.5; 5.7 3.2;6 _1.1; 6 2.942

   NB. Combine a and b with each point in turn, and find the areas
   NB. of all triangles so formed
   A=. [: area"2 ,"2 1
   base=. [: +/&.*: -/
   NB. Double the area and divide by the length of the base
   d=. +:@] % base@[                NB. Distance to the line

   NB. Append the sign of the area and the distance
   NB. Classify by sorting output by the signed size of the distance
   class=. \:@A { ] ,"1 *@A ,. [ d A
   class
   NB. First note the conjunctions, and then count the number of forks

   z=.(a,:b) class c,p,-:a+b
   5.2 9.3 3 9.4": z                NB. Format the table

   NB. COMPLEX NUMBERS: EASY AND USEFUL
   2&* 1 2             NB. Double
   _1&* 1 2            NB. Reverse
   g=. ] * [: %: 2:
   g 1 2
   g g 1 2             NB. Applied twice, g doubles
   h=. ] * [: %: _1:
   h h 1 2             NB. Applied twice, h reverses

   p=. 1j2             NB. 1 on x-axis, 2 on y-axis
   2&* p               NB. Double
   _1&* p              NB. Reverse
   g p
   g g p               NB. Applied twice, g doubles
   h p
   h h p               NB. Applied twice, h reverses

   1ad60               NB. Unit vector at angle of 60 degrees
   2ar1                NB. Unit vector at angle of 1 radian
   +. 1ad60            NB. Separate the two parts
   r. rfd 60           NB. Coordinates of unit vectors
   2 r. rfd 60         NB. Polar to Cartesian coordinates
   +/&.*:@+. 3j4       NB. Pythagoras

   NB. AREA OF A POLYGON
   a=. 3%:_1           NB. To reverse 1j0 to _1j0 in three steps
   +. a
   +. a^0
   +. a^3              NB. Three steps takes us half way round
   ,. +. a^i.6         NB. Six steps completes the hexagon

   poly=. (-: %: _1:) ^ i.     NB. The general case
   polygon=. ,.@+.@poly
   hex=. polygon 6             NB. Hexagon
   polygon 4                   NB. Square
   polygon 5                   NB. Pentagon
   p=. 4 _3                    NB. Arbitrary point
   hex , {. hex                NB. Cycle must be complete
   y=. 2 <\ (, {.) hex         NB. The 6 sides of the hexagon
   each=. &.>                  NB. (Conjunction - Verb) is Adverb
   p&,each y                   NB. Complete 6 triangles

   NB. The sum of the signed areas of the triangles is the polyogon area
   +/ > area each p&,each 2 <\ (, {.) hex
   +/ > area&.> p&,&.> 2 <\ (, {.) hex

   NB. Alternative using the rank conjunction
   +/area"2 (2) p&,"1 2\ (, {.) hex

   s=. '+/ > area each p&,each 2 <\ (, {.) y.'
   e=. 3 : s            NB. Explicit definition
   e hex
   T=. 13 : s           NB. Tacit definition
   linear 'T'

   NB. Build the triangles by composing functions
   h=. 2: <\ ] , {.
   p&,&.> h hex

   NB. To change p, it must be freed from the verb to which it is bonded
   g=. [ ,&.> h@]         NB. p can be an argument to this fork
   p=. <4 _3.             NB. p must be boxed for agreement
   p g hex         NB. Boxed coordinates of the corners of the triangles

   NB. Open the boxes and find the areas; then sum the areas
   f=.  area &.>@g
   poly=. [: +/ >@f
   p poly hex

   NB. Try a series of arbitrary points
   p=. <"1 i. 6 3 2
   $ h hex
   $ p g hex
   p g hex             NB. This is the general case

   NB. We need the special case. Values in a given column are the same
   p=. 6 3$ 4 _3; 0 0; 1 2
   p g hex

   NB. The numbers in the middle column are all the same because
   NB. all corners are equidistant from the origin
   area each p g hex
   +/ > area each p g hex    NB. All arbitrary points give same area

   NB. Now compose a function
   poly=. [: +/ >@(area each@g)
   (>area) each@g
   poly=. [: +/ [: > area&.>@g
   p poly hex 
   NB. The area is unaffected by rotation
   rot=. 2 1&o. ,: _1 1"_ * 1 2&o.
   rotate=. rot@rfd    NB. Rotation matrix.  Angle in degrees

   NB. Apply this rotation and make an arbitrary translation
   setrl=. 9!:1
   setrl 7^5        NB. Set random link to standard initial value
   y=. 100 %~ ?10000
   2 3 +"1 hex +/ .* rotate y
   p poly hex      NB. All rotations give the same area

   NB. LINE OF INTERSECTION OF TWO PLANES
   m=. 1 _2 4 6,: 2 1 _3 8    NB. Define two planes
   n=. |: m                   NB. Transpose

   k=. 1 2, 2 0,: 0 1
   h=. det@(k&{"1 2)@}:
   h n                NB. Direction numbers of line of intersection
   (h n) % rss h n    NB. Normalize to get direction cosines
   norm=. % rss       NB. A hook
   dc=. norm@h        NB. Direction cosines

   NB. Coordinates of a point on the line
   ct=. 0: ,~ {: %. |:@(2&{.)@}:
   ct n

   (9!:1) 7^5                   NB. Set random link
   ]t=. 0.1* 50-~?5#100         NB. Choose 5 random parameters

   NB. Coordinates of 5 random points on the line of intersection
   ]z=. (ct n) +"1 (dc n) *"1 0 t

   pequ=. ct@[ +"1 dc@[ *"1 0 ]   NB. Parametric equation of the line
   z-: n pequ t                   NB. These match
   NB. Verify that these points satisfy the equations of both planes
   (n pequ t) +/ .* }:n

   normal=. norm@}:"1     NB. Find the normals to the two planes
   normal m

   NB. On each plane choose 2 arbitrary points.
   NB. The line joining any 2 points on the plane is perpendicular
   NB. to the normal
   (8 1 0-2 0 1) +/ .* normal {.m
   (1 6 0- 5.5 0 1) +/ .* normal{:m

   NB. The perpendicular distances from the origin to each plane
   dist=. ({: % rss@}:)"1
   dist m

   NB. The points where the normals from the origin meet the planes
   (dist * normal) {.m
   (dist * normal) {: m

   NB. Proof that these points lie on their respective planes
   1 _2 4 +/ .* (dist * normal) {.m
   2 1 _3 +/ .* (dist * normal) {:m

   NB. Angle between the planes is the angle between their normals
   'ab'=. (normal {.m);normal {:m
   dfr _2 o. a +/ .* b
   dfr _2 o. a +/ .* -b

   NB. The two normals define a plane perpendicular to the line of
   NB. intersection
   u=. 1&|.@[ * _1&|.@]
   v=. _1&|.@[ * 1&|.@]
   vcp=. u - v     NB. Vector cross-product is line of intersection
   c=. a vcp b
   c +/ .* a,.b    NB. c is normal to a-b

   NB. Direction cosines of the line of intersection in two ways
   (dc n) -: norm c         NB. These match

   NB. PROJECTION OF A LINE ONTO A GIVEN PLANE
   NB. vector p makes angles of 110 80 22 to the Cartesian axes
   ]p=. 2 o. rfd 110 80 22
   NB. Project p onto the plane x - 2y + 4z = 6, whose normal is a
   NB. q is normal to the plane p-a
   q=. norm p vcp a
   NB. r is the normal to plane a-q. It is the required projection of p
   r=. norm a vcp q
   ]z=. norm a vcp norm p vcp a
   NB. The required projection is a train of 5 verbs
   proj=. [: norm ] vcp norm@vcp
   z -: p proj a      NB. These match

   NB. VOLUME OF A PARALLELEPIPED
   (;:'a b c'),: ;:'alpha beta gamma'
   <"0 ch=. 6.11 10.673 5.95,: 97.583 107.167 77.55
   sin=. 1&o.
   cos=. 2&o.
   cosd=. cos@rfd

   erase 'h p q'

   volume=. */@{. * %:@h@cosd@{:
   NB. h is as yet undefined. It is assumed it will bedefined as a verb
   h=. p - q
   p=. >:@+:@(*/)       NB. 1+ twice the product
   q=. +/@*:            NB. Sum of squares
   volume ch

   NB. h is equivalent to the determinant given by g
   g=. (0 3 2, 3 0 1,: 2 1 0)"_ { 1&,
   g
   h=. det@g
   volume=. */@{. * %:@h@cosd@{:
   volume ch

   NB. Axial lengths or (depending on context) interacial angles
   a=. 0&{ [.  b=. 1&{ [.  c=. 2&{
   axisa=. sin@b , 0: , cos@b

   NB. Alternative forms
   CosRho=. (cos@c - */@cos@(a,b)) % sin@b
   CosRho=. (cos@c - [: */@cos a,b) % sin@b
   CosRho=. sin@b %~ cos@c - [: /@cos a,b
   CosRho=. sin@b %~ cos@c - cos@a * cos@b

   CosSigma=. sin@b %~ (>:@+:@(*/@cos) - +/@*:@cos)
   CosSigma=. sin@b %~ [: %: >:@+:@(*/@cos) - +/@*:@cos

   axisb=. CosRho,CosSigma,cos@a
   dm=. ,&0 0 1 @(axisa,:axisb)@rfd@b
   dm=. 0 0 1"_ ,~ (axisa,:axisb)@rfd@b
   dmat=. ({. *"0 1 dm)"2     NB. Matrix defining the direct lattice
   ]d=. dmat ch               NB. The "dmatrix"

   det d           NB. Volume of the cell
   NB. Bordering the dmatrix gives a signed volume
   det (,"1 1:) d,0
   det (,"1 1:) 0,d
   e=. 1 _2 3 +"1 d,0    NB. After an arbitrary translation
   NB. the matrix must be bordered in order to get the volume
   det (,"1 1:) e

   NB. The transpose of the inverse is the inverse of the transpose
   cl=. * 1e_15&<@|                 NB. Clean very small values
   (cl |:@%. d)  -:  cl %.@|: d     NB. These match
   ch=. chalcanthite=. 6.11 10.673 5.95,: 97.583 107.167 77.55
   or=. orthoclase=. 8.562 12.996 7.193,: 90 116.01 90
   an=. anorthite=. 8.177 12.877 14.169,: 93.17 115.85 81.22
   ax=. axinite=. 7.15 12.57 13.05,: 91.383 75.5 93.383
   ky=. kyanite=. 7.12 7.85 5.57,: 89.983 101.117 106
   $minerals=. ch,or,an,ax,:ky
   $x=. dmat minerals
   dmat b. 0       NB. dmat is monadic and has rank 2
   %. b. 0         NB. monadic matrix-inverse has rank 2
   |: b. 0         NB. monadic transpose has unbounded rank
   NB. The reciprocal lattice is defined by the transpose of the
   NB. inverse of dmat
   rmat=. |:@%.
   |:@%. b. 0
   %.@|: b. 0
   $ rmat x
   NB. Atempted execution of the following give length errors
   NB. %. |: x
   NB. %.@|: x
